package com.example.streaminputttsdemoproj;

import android.app.Activity;
import android.graphics.Point;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.Display;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import com.alibaba.fastjson.JSONException;
import com.alibaba.fastjson.JSONObject;
import com.alibaba.idst.nui.Constants;
import com.alibaba.idst.nui.INativeStreamInputTtsCallback;
import com.alibaba.idst.nui.NativeNui;
import java.util.Arrays;

// 本样例展示在线语音合成（流式输入流式输出）使用方法
// Android SDK 详细说明：
public class MainActivity extends Activity {
    final static String demoText =
            "流式文本语音合成SDK，\n可以将输入的文本\n合成为语音二进制数据，\n相比于非"
                    + "流式语音合成，\n流式合成的优势在于实时性\n更强。用户在输入文本的同时\n可"
                    + "以听到接近同步的语音输出，\n极大地提升了交互体验，\n减少了用户等待时间。"
                    + "\n适用于调用大规模\n语言模型（LLM），以\n流式输入文本的方式\n进行语音合成"
                    + "的场景。";
    private static final String TAG = "StreamInputTtsBasicActivity";

    private Button ttsStartBtn;
    //  AudioPlayer默认采样率是16000
    private final AudioPlayer mAudioTrack =
            new AudioPlayer(new AudioPlayerCallback() {
                @Override
                public void playStart() {
                    Log.i(TAG, "start play");
                }

                @Override
                public void playOver() {
                    Log.i(TAG, "play over");
                    runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            ttsStartBtn.setEnabled(true);
                        }
                    });
                }
            });
    private EditText inpuTextView;
    private EditText logTextView;
    private String ttsText = new String();

    /**
     * 配置前端控件
     */
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        Display display = getWindowManager().getDefaultDisplay();
        Point size = new Point();
        display.getSize(size);
        int screenHeight = size.y;

        // logTextView用于输出log信息
        logTextView = (EditText) findViewById(R.id.logTextView);
        logTextView.setHeight(screenHeight / 3); // 设置高度
        // inputTextField用于显示测试文本，可以输入为您希望合成的内容
        inpuTextView = (EditText) findViewById(R.id.inputTextView);
        inpuTextView.setHeight(screenHeight / 3); // 设置高度
        inpuTextView.setText(demoText);
        // 开始按钮，点击后开始执行一次流式tts任务
        ttsStartBtn = (Button) findViewById(R.id.buttonStart);
        ttsStartBtn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                // 音频播放器开始
                mAudioTrack.play();
                // 由于需要开启线程播放，因此按钮按下后禁用，本DEMO只做一次流式语音合成任务
                ttsStartBtn.setEnabled(false);
                // 创建新线程，并完成一次流式语音合成任务
                Thread sendThread = new Thread() {
                    public void run() {
                        try {
                            doStreamInputTts(demoText);
                        } catch (Exception e) {
                            System.out.println(e);
                        }
                    }
                };
                sendThread.start();
            }
        });
        // 开启播放器，默认使用24000采样率
        mAudioTrack.setSampleRate(24000);
    }

    private void doStreamInputTts(String inputContents)
            throws InterruptedException {
        // 获得鉴权信息和语音合成参数
        String ticket = genTicket();
        String parameters = genParameters();
        // 获得流式TTS实例，并配置callback
        NativeNui streamInput_tts_instance =
                new NativeNui(Constants.ModeType.MODE_STREAM_INPUT_TTS);
        INativeStreamInputTtsCallback callback =
                new INativeStreamInputTtsCallback() {
                    /**
                     * 事件回调，打印出服务返回的完整信息，可以根据需要处理对应的事件类型
                     */
                    @Override
                    public void onStreamInputTtsEventCallback(
                            INativeStreamInputTtsCallback.StreamInputTtsEvent event,
                            String task_id, String session_id, int ret_code,
                            String error_msg, String timestamp, String all_response) {
                        Log.i(TAG,
                                "stream input tts event:" + event + " session id "
                                        + session_id + " session id " + task_id + " ret "
                                        + ret_code);
                        switch (event) {
                            case STREAM_INPUT_TTS_EVENT_SYNTHESIS_STARTED:
                                // TODO: 处理SynthesisStarted指令
                                break;
                            case STREAM_INPUT_TTS_EVENT_SENTENCE_BEGIN:
                                // TODO: 处理SentenceBegin指令
                                break;
                            case STREAM_INPUT_TTS_EVENT_SENTENCE_SYNTHESIS:
                                // TODO: 处理SentenceSynthesis指令
                                break;
                            case STREAM_INPUT_TTS_EVENT_SENTENCE_END:
                                // TODO: 处理SynthesisEnd指令
                                break;
                            case STREAM_INPUT_TTS_EVENT_SYNTHESIS_COMPLETE:
                                // TODO: 处理SynthesisComplete指令
                                break;
                            case STREAM_INPUT_TTS_EVENT_TASK_FAILED:
                                // TODO: 处理TaskFailed指令
                                break;
                            default:
                                break;
                        }
                    }

                    /**
                     * 音频回调函数，将收到的音频帧写入到音频播放器中
                     */
                    @Override
                    public void onStreamInputTtsDataCallback(byte[] data) {
                        if (data.length > 0) {
                            mAudioTrack.setAudioData(data);
                        }
                    }
                };
        // 建立连接并开始语音合成任务
        // 其中callback为包含配置好回调的INativeTtsCallback对象
        int ret = streamInput_tts_instance.startStreamInputTts(
                callback, ticket, parameters, "", 0, false);
        String[] lines = inputContents.split("\n");
        for (int i = 0; i < lines.length; i++) {
            // 取第一行文本发送
            ret = streamInput_tts_instance.sendStreamInputTts(lines[i]);
            // 更新文本内容，去除第一行文本
            final String remainingText = TextUtils.join(
                    "\n", Arrays.copyOfRange(lines, i + 1, lines.length));
            runOnUiThread(() -> inpuTextView.setText(remainingText));
            final String sendedText = "已发送文本：\n"
                    + TextUtils.join("\n", Arrays.copyOfRange(lines, 0, i));
            runOnUiThread(() -> logTextView.setText(sendedText));
            // 模拟LLM合成的文本，两次发送间间隔一秒
            Thread.sleep(1000);
        }
        // 停止发送文本，阻塞至所有音频返回，并结束本次流式语音合成
        ret = streamInput_tts_instance.stopStreamInputTts();
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        mAudioTrack.stop();
        mAudioTrack.releaseAudioTrack();
    }

    /**
     * 获取入会所需鉴权参数,
     * 代码中"your-appkey"以及"your-token"字符串变量需要您自行填入，url默认使用北京服务
     */
    private String genTicket() {
        String str = "";
        try {
            JSONObject object = new JSONObject();
            object.put("appkey", "<您申请创建的app_key>"); // 必填
            object.put("token", "<服务器生成的具有时效性的临时凭证>"); // 必填
            object.put("url",
                    "wss://nls-gateway.cn-shanghai.aliyuncs.com:443/ws/v1"); // 默认
            object.put("complete_waiting_ms", "10000"); // 默认
            str = object.toString();
        } catch (JSONException e) {
            e.printStackTrace();
        }
        Log.i(TAG, "UserContext:" + str);
        return str;
    }

    /**
     * 语音合成相关的参数，默认使用24000 采样率。
     */
    private String genParameters() {
        String str = "";
        try {
            JSONObject object = new JSONObject();
            object.put("voice", "longxiaochun");
            object.put("format", "wav");
            object.put("sample_rate", "24000");
            object.put("volume", "50");
            object.put("speech_rate", "0");
            object.put("pitch_rate", "0");
            object.put("enable_subtitle", "0");
            object.put("session_id", "");
            str = object.toString();
        } catch (JSONException e) {
            e.printStackTrace();
        }
        Log.i(TAG, "UserContext:" + str);
        return str;
    }
}
